#!/bin/sh

set -e

PROGDIR=/usr/share/rootless-helper-astra

if [ "$EUID" -ne 0 ]; then
    printf 'rootless-docker-setup: this script must be executed with root privileges\n' 1>&2
    exit 1
fi

CONFIG_FILE=/etc/rootless-helper-astra.conf
if test -f $CONFIG_FILE; then
    source $CONFIG_FILE
fi
if [ -z $ROOTLESS_DOCKER_ROOT ]; then
    ROOTLESS_DOCKER_ROOT=/var/lib/rootlessdocker
fi

start_without_label() {
    local username=$1
    pdp-exec -u $username -- $PROGDIR/start-nolabel.sh
}

set_ccnr() {
    local dir=$1
    local sysmaxlev=3
    local sysmaxilev=`cat /sys/module/parsec/parameters/max_ilev`
    local sysmaxcat=0xffffffffffffffff
    local sysmaxlbl="$sysmaxlev:$sysmaxilev:$sysmaxcat"
    local real_dir=$(readlink -e $dir)
    pdpl-file "$sysmaxlbl:CCNRA" $real_dir
}

setup_user_folder() {
    local username=$1
    mkdir -p "$ROOTLESS_DOCKER_ROOT/$username"
    set_ccnr "$ROOTLESS_DOCKER_ROOT/$username"
    chown -R $username:$username "$ROOTLESS_DOCKER_ROOT/$username"
}

parse_arg() {
    local arg=$1
    IFS='@'
    local arg_splitted
    read -ra arg_splitted <<< "$arg"
    username=${arg_splitted[0]}
    pdp_label=${arg_splitted[1]}
    flags=${arg_splitted[2]}
    if [ -z $username ]; then
        printf 'Expected argument format: <user>[@<pdp_label>[@<flags>]]\n' 1>&2
        exit 1
    fi
}

if [ $# -ne 1 ]; then
    printf 'Only one argument should be passed\n' 1>&2
    exit 1
fi

parse_arg $1

systemctl start user@$(id -u $username)

if [ -z $pdp_label ]; then
    # Fallback to legacy start (without label).
    # Use current subject label in this case.
    lev=$(pdp-id -l)
    ilev=$(pdp-id -i)
    cat=$(pdp-id -c)
    pdp_label=$lev:$ilev:$cat
fi



setup_user_folder $username
unshare -m $PROGDIR/unshare.sh $ROOTLESS_DOCKER_ROOT $username $pdp_label $flags
