#!/bin/sh

set -e

if [ $# -ne 3 ] && [ $# -ne 4 ]; then
    printf 'Invalid arguments\n' 1>&2
    exit 1
fi

PROGDIR=/usr/share/rootless-helper-astra
ROOTLESS_DOCKER_ROOT=$1
username=$2
pdp_label=$3
flags=$4
lev=$(pdp-exec -l $pdp_label pdp-id -l)
ilev=$(pdp-exec -l $pdp_label pdp-id -i)
cat=$(pdp-exec -l $pdp_label pdp-id -c)
IFS=','
read -ra flags_array <<< "$flags"
flags_array_len=${#flags_array[@]}
cap=0x80000
for (( i=0; i<$flags_array_len; i++ )); do
    if [[ ${flags_array[$i]} == "privsock" ]]; then
        cap=0x80100 # turn on PARSEC_CAP_PRIV_SOCK
    else
        printf "Invalid flag: '%s'\n" "${flags_array[$i]}"
        exit 1
    fi
done

workdir="$ROOTLESS_DOCKER_ROOT/$username/l${lev}i${ilev}c${cat}t0x0"
mkdir -p "$workdir"
pdpl-file $pdp_label "$workdir"
mkdir -p "$workdir/tmp"
pdpl-file $pdp_label "$workdir/tmp"
run_name="rootless-run"
mkdir -p "$workdir/$run_name"
pdpl-file $pdp_label "$workdir/$run_name"
chown -R $username:$username "$workdir"

pdp-exec -l $pdp_label -u $username -- mkdir -p "$workdir/tmp/.X11-unix"
pdp-exec -l $pdp_label -u $username -- mkdir -p "$workdir/tmp/Downloads"

mount --rbind /tmp/.X11-unix "$workdir/tmp/.X11-unix"

# Set /tmp/Downloads folder in rootless Docker's mount namespace to
# host's /home/$USER/Downloads.
# The label of this folder will be set to 0:0
# (regardless of strict_mode ON/OFF)
# Rootless Docker will later use it as a volume which is mounted as
# /root/Downloads inside the guest so browsers will transparently
# download files to host's Downloads folder.

downloads_host_dir=/home/$username/Downloads
mkdir -p $downloads_host_dir

# check if downloads_host_dir's label is 0:0. If not, set it to 0:0
d_lbl=$(pdp-ls -Mdn $downloads_host_dir | awk '{print $5}')
d_lev=$(echo $d_lbl | awk -F ':' '{print $1}')
d_ilev=$(echo $d_lbl | awk -F ':' '{print $2}')
d_cat=$(echo $d_lbl | awk -F ':' '{print $3}')
if [[ $d_lev -ne 0 || $d_ilev -ne 0 || $d_cat -ne 0x0 ]]; then
    pdpl-file -Rr 0:0 $downloads_host_dir
fi

mount --rbind "$downloads_host_dir" "$workdir/tmp/Downloads"

mount --make-rslave /

mount --rbind "$workdir/tmp" /tmp

user_id=$(id -u $username)
bus_file="$workdir/$run_name/bus" 
sudo rm -f $bus_file
ln -s /run/user/$user_id/bus $bus_file

pdp-exec -l $pdp_label -u $username -c $cap -- $PROGDIR/start-label.sh \
    $ROOTLESS_DOCKER_ROOT $username
